#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create a package containing file /foo
# 2. Create an identically named/versioned package containing file /foo and /bar
# 3. Install the first package.
# 4. Force reinstall the second package from a repository (passing the package
#    on the command line will not show the bug).
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# The package should show that it owns both /foo and /bar. It only shows
# ownership of the file(s) that were installed initially.
#
# $ ./opkg -o /tmp/opkg files a
# Package a (1) is installed on root and has the following files:
# /tmp/opkg/foo
#
# If the package is now removed, /bar will be left on the system.
#
#
# Additional
# ==========
#
# Also a problem, is when the --force-reinstall'd package contains fewer files.
# These files are not properly orphaned/deleted.
#
# Perhaps --force-reinstall should do a --force-depends remove, then install?
#
#
# Status
# ======
#
# Graham Gower:
# > Fixed with r538.

import os
import opk, cfg, opkgcl

opk.regress_init()

open("foo", "w").close()
a1 = opk.Opk(Package="a")
a1.write(data_files=["foo"])
os.rename("a_1.0_all.opk", "a_with_foo.opk")

opkgcl.install("a_with_foo.opk")

# ----
opkgcl.install("a_with_foo.opk")

open("bar", "w").close()
o = opk.OpkGroup()
a2 = opk.Opk(Package="a")
a2.write(data_files=["foo", "bar"])
o.opk_list.append(a2)
o.write_list()

os.unlink("foo")
os.unlink("bar")

opkgcl.update()
opkgcl.install("a", "--force-reinstall")

foo_fullpath = "{}/foo".format(cfg.offline_root)
bar_fullpath = "{}/bar".format(cfg.offline_root)

if not os.path.exists(foo_fullpath) or not os.path.exists(bar_fullpath):
    opk.fail("Files foo and/or bar are missing.")

a_files = opkgcl.files("a")
if not foo_fullpath in a_files or not bar_fullpath in a_files:
    opk.fail("Package 'a' does not own foo and/or bar.")

opkgcl.remove("a")

if os.path.exists(foo_fullpath) or os.path.exists(bar_fullpath):
    opk.fail("Files foo and/or bar still exist after removal of package 'a'.")

# ----
o = opk.OpkGroup()
a2 = opk.Opk(Package="a")
a2.write()
o.opk_list.append(a2)
o.write_list()


opkgcl.update()

opkgcl.install("a", "--force-reinstall")

if os.path.exists(foo_fullpath):
    opk.fail("File 'foo' not orphaned as it should be.")

if foo_fullpath in opkgcl.files("a"):
    opk.fail("Package 'a' incorrectly owns file 'foo'.")

opkgcl.remove("a")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Install a package 'x' which PROVIDES 'v'. Then try to install 'y' which
# PROVIDES and CONFLICTS 'v', indicating that no other provider of the virtual
# package 'v' should be installed at the same time as 'y'.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="x", Provides="v")
o.add(Package="y", Provides="v", Conflicts="v")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("x")
if not opkgcl.is_installed("x"):
    opk.fail("Package 'x' installed but reports as not installed.")

# Now try to install "y", which should fail

opkgcl.install("y")
if opkgcl.is_installed("y"):
    opk.xfail("[internalsolv] Package 'y' installed despite conflict with 'v' provided by 'x'.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create a package, A.opk, with unresolved dependencies.
# 2. Create another package, B.opk, that depends upon A.
# 3. opkg install B.opk
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# A is installed and can be found in the status file.
#
#
# Status
# ======
#
# Graham Gower:
# > Can no longer reproduce. Must have been fixed as a side effect of other
# > changes...

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b", Depends="c")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if opkgcl.is_installed("a"):
    opk.fail("Package 'a' installed, despite dependency "
             "upon a package with an unresolved dependency.")

if opkgcl.is_installed("b"):
    opk.fail("Package 'b' installed, despite unresolved dependency.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# What steps will reproduce the problem?
# ======================================
#
# Create two packages, with differing versions. Provide the newer package in an
# accessible repository. For this test I manually created an older
# abiword_1.6.8-r0.3 package (identical contents except the control file).
#
# $ ./opkg -o ~/opkg install abiword
# Installing abiword (2.6.8-r0.3) to root...
# Downloading http://10.0.0.13/ipk/mipsel/abiword_2.6.8-r0.3_mipsel.ipk
# Configuring abiword
# sh: rm: command not found
#
# $ ./opkg -o ~/opkg -force-downgrade install ~/opkg/abiword_1.6.8-r0.3_mipsel.ipk
# Downgrading abiword on root from 2.6.8-r0.3 to 1.6.8-r0.3...
# Configuring abiword
# sh: rm: command not found
#
# $ ./opkg -o ~/opkg -force-downgrade install ~/opkg/abiword_1.6.8-r0.3_mipsel.ipk
# Installing abiword (2.6.8-r0.3) to root...
# Downloading http://10.0.0.13/ipk/mipsel/abiword_2.6.8-r0.3_mipsel.ipk
# Configuring abiword
# sh: rm: command not found
#
# Note that it is now impossible to install the older version with opkg. To
# recover from this situation, the package information must be deleted from
# /usr/lib/opkg/status manually.
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# I expect the installation of the specified package to fail if it is already
# installed.
#
#
# Status
# ======
#
# Graham Gower:
# > I can no longer reproduce this. I suspect it has been fixed as a side effect
# > of r465.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Version="2.0")
o.write_opk()
o.write_list()

# older version, not in Packages list
a1 = opk.Opk(Package="a", Version="1.0")
a1.write()

opkgcl.update()

# install v2 from repository
opkgcl.install("a")
if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a_2.0' not installed.")

opkgcl.install("a_1.0_all.opk", "--force-downgrade")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a_1.0' not installed (1).")

opkgcl.install("a_1.0_all.opk", "--force-downgrade")
if not opkgcl.is_installed("a", "1.0"):
    opk.fail("Package 'a_1.0' not installed (2).")

opkgcl.remove("a")



#!/usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: ovidiu.vancea@ni.com
#
# What steps will reproduce the problem?
########################################
#
# 1.- Create package 'a' (version 1.0), which Recommends 'b (= 1.0)'
# 2.- Create package 'b' (version 1.0)
# 3.- Install 'a' (b 1.0 will also be installed)
# 4.- Create package 'a' (version 2.0), which Recommends 'b (= 2.0)'
# 5.- Create package 'b' (version 2.0)
# 6.- Upgrade package 'a' (opkg upgrade a)
#
# What is the expected output? What do you see instead?
#######################################
#
# Both package 'a' and 'b' should be upgraded to version 2.0.
# Instead, only package 'a' is upgraded.
#

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()

o.add(Package='a', Recommends='b (= 1.0)', Version='1.0')
o.add(Package='b', Version='1.0')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install('a')

o = opk.OpkGroup()

o.add(Package='a', Recommends='b (= 2.0)', Version='2.0')
o.add(Package='b', Version='2.0')
o.write_opk()
o.write_list()

opkgcl.update()
opkgcl.upgrade('a')

if not opkgcl.is_installed("a", "2.0"):
    opk.fail("Package 'a' failed to upgrade")

if not opkgcl.is_installed("b", "2.0"):
    opk.fail("Package 'b' failed to upgrade")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: Graham Gower
#
# What steps will reproduce the problem?
# ======================================
#
# 1. Create package A which recommends B.
# 2. Create package B.
# 3. Install A.
# 4. Remove A with --autoremove
#
#
# What is the expected output? What do you see instead?
# =====================================================
#
# Expect B to be removed with A. Instead only A is removed.
#
#
# Status
# ======
#
# Fixed with r555/r556.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Recommends="b")
o.add(Package="b")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")

opkgcl.remove("a", "--autoremove")
if opkgcl.is_installed("b"):
    opk.fail("Recommended package 'b' not autoremoved.")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reporter: markus.lehtonen@linux.intel.com
#
# What steps will reproduce the problem?
########################################
#
#  1.- Create package 'a', which RECOMMENDS 'b'
#  2.- Create package 'b'
#  3.- Create package 'c', which DEPENDS on 'd'
#  4.- Create package 'd', which CONFLICTS with 'b'
#  5.- Install 'a'
#  6.- Install 'c'
#
# What is the expected output? What do you see instead?
########################################
#
#  Package 'a', 'c' and 'd' should be installed (and b should
#  be removed). Instead the installaton of 'c' fails.
#

import os
import opk, cfg, opkgcl
import re

opk.regress_init()

o = opk.OpkGroup()
o.add(Package='a', Recommends='b')
o.add(Package='b')
o.add(Package='c', Depends='d')
o.add(Package='d', Conflicts='b')
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install('a')
if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' not installed but reports as being installed.")
if not opkgcl.is_installed('b'):
    opk.fail("Package 'b' not installed but reports as being installed.")

opkgcl.install('c')
if not opkgcl.is_installed('c'):
    opk.xfail("[internalsolv] Package 'c' not installed but reports as being installed.")
if not opkgcl.is_installed('d'):
    opk.fail("Package 'c' not installed but reports as being installed.")
if not opkgcl.is_installed('a'):
    opk.fail("Package 'a' was incorrectly removed.")
if opkgcl.is_installed('b'):
    opk.fail("Package 'b' was not removed, but conflicts with 'd'")



#! /usr/bin/env python3
# SPDX-License-Identifier: GPL-2.0-only
#
# Reported by alejandro.delcastillo@ni.com
#
# What steps will reproduce the problem?
########################################
#   1. Create package 'a', which DEPENDS on 'b'
#   2. Create package 'b', which DEPENDS on 'a'
#   3. Install 'a'
#
# What is the expected output? What do you see instead?
#######################################################
#   I expect package 'a' and 'b' to be installed, and only package 'b' to be set as Autoinstalled.
#   Instead, both package 'a' and 'b' are set to Autoinstalled.

import os
import opk, cfg, opkgcl

opk.regress_init()

o = opk.OpkGroup()
o.add(Package="a", Depends="b")
o.add(Package="b", Depends="a")
o.write_opk()
o.write_list()

opkgcl.update()

opkgcl.install("a")
if not opkgcl.is_installed("a"):
        opk.fail("Package 'a' installed but does not report as installed.")
if not opkgcl.is_installed("b"):
        opk.fail("Package 'b' should be installed as a dependency of 'a' but does not report as installed.")
if opkgcl.is_autoinstalled("a"):
        opk.fail("Package 'a' explicitly installed by reports as auto installed.")
if not opkgcl.is_autoinstalled("b"):
        opk.fail("Package 'b' installed as a dependency but does not report as auto installed.")
